__all__ = [
    "user",
    "role",
    "permission",
    "news_item",
    "collectors_node",
    "collector",
    "osint_source",
    "parameter",
    "parameter_value",
    "report_item",
    "attribute",
    "report_item_type",
    "address",
    "organization",
    "acl_entry",
    "product_type",
    "product",
    "presenters_node",
    "presenter",
    "publishers_node",
    "publisher_preset",
    "publisher",
    "word_list",
    "asset",
    "bot",
    "bots_node",
    "bot_preset",
    "notification_template",
    "remote",
    "tag_cloud",
    "token_blacklist",
]
